<?php

/*
*** Arguments (via GET method) *** PROFILE=profilename Required,
PREFIX must contain a file profilename.conf with KEY=VALUE pairs, one
per line.

NOCURL=1 Optional, if set, we rerun the script using the last data
from sprintpcs again. Good for testing without hammering on sprint's
web site so much.

DEBUG=1 Optional. If set, displays human readable progress information
and a dump of the data parsed from Sprint.

NOWRITE=1 Optional. If set, no output files will be updated.

*** Configuration file directives ***
PHONE=1234567890
PASSWORD=mypassword
*/

include ("sprintpcscommon.php");

$debug=getparam_bool("DEBUG",true);
$nocurl=getparam_bool("NOCURL",false);
$nowrite=getparam_bool("NOWRITE",false);

$profile=getparam_string("PROFILE",null);
if ($profile==null)
  fail("No profile specified");

$basepath=$SPRINTPREFIX.$profile;
$configpath=$basepath.".conf";

$conf=readconfig($configpath);
if ($conf==null)
  fail("Couldn't read profile ".$configpath);

if (!array_key_exists("PHONE",$conf))
  fail("No PHONE specified in $configpath");
if (!array_key_exists("PASSWORD",$conf))
  fail("No PASSWORD specified in $configpath");

if ($debug)
  print "<h2>Retrieving current data from SprintPCS</h2>";

if ($nocurl)
{
  print "Using previous web request data.<br>";
}
else
{
  // make some white space to help browsers decide to start drawing the page.
  if ($debug)
    {
      for ($i=0;$i<100;$i++)
	print "      ";
      
      flush();
    }

  $postdata="min=$conf[PHONE]&password=$conf[PASSWORD]&action=doLogin&target=Login&signIn_x=0&signIn_y=0";
  $referrer="https://manage1.sprintpcs.com/Manage";
  $url="https://manage1.sprintpcs.com/Manage";
  $header=$basepath."headers.log";
  $outputfile=$basepath."ph1.html";

  $inittime=time();
  if ($debug)
    {
      print "<pre>Beginning at time ".date("r",$inittime)."<br>";
      print "First phase...<br>";
      flush();
    }

  $cmd="curl -s -L --dump-header $header -d \"$postdata\" \"$url\" -e \"$referrer\" -o $outputfile";
  //  print $cmd."\n";
  passthru($cmd);
  flush();

  $url="https://manage1.sprintpcs.com/Manage?layer=0&target=MyCurrentUsage&action=current_usage";

  $outputfile=$basepath."ph2.html";
  
  if ($debug)
    {
    print "Second phase...\n";
    flush();
    }

  $cmd="curl -s -L -b $header -e \"$referrer\" -o $outputfile \"$url\"";
  //  print $cmd."\n";
  passthru($cmd);

  if ($debug) 
    {
      $completetime=time();
      print "Finished at ".date("r",$completetime)." (elapsed time ".($completetime-$inittime)." seconds)<br>";
      print "</pre>";

    }
}

$response=file($basepath."ph2.html");

$sprintdata=array();
$sprintdataidx=0;

/** Check each line for something that looks like a minutes status line,
 e.g., OFF-PEAK MINUTES  1000 50 950 0 */
for ($i=0;$i<sizeof($response);$i++)
{
  //  print $response[$i];
  preg_match("/>([A-Z\-\ ]{4,})<[^0-9]*([0-9]+)[^0-9]*([0-9]+)[^0-9]*([0-9]+)[^0-9]*([0-9]+)/",$response[$i], $matches);

  if (sizeof($matches)>0)
    {
      // We got a line of actual minutes usage! Build a useful array
      // The numbers should be organized as:
      // 2:plan  3:used  4:remaining  5:overage
      $temp=array();
      $temp["plan"]=$matches[2];
      $temp["used"]=$matches[3]+$matches[5]; // really used=used+overage

      // do some sanity checking. plan-remaining+overage=used Sprint's
      // website (as of 8/12/02) currently does not correctly report
      // overage data for people with retention plans. (Rentention
      // plans often have *multiple* MINUTES IN PLAN fields for one
      // telephone number.)  presumably, there's a bug of some sorts
      // in sprint's system. The following sanity check has been
      // disabled because of this; it fails.
      // sample data:
      //
      // data pulled from eolson's account, 8/12/02.
      // error confirmed by dlamming.
      //
      //                 plan used remain overage
      // MINUTES IN PLAN 150     0    150    0
      // MINUTES IN PLAN 500   508      0    0
      // OFF-PEAK OPTION 2000  126   1874    0
      // OFF-PEAK OPTION 1000    0   1000    0
      //
      // Note how sprint has not properly started consuming minutes
      // out of the 150 minute field; 8 used minutes are simply
      // unaccounted for.  I believe that if usage exceeded 650 that
      // sprint would properly bill for it, but I do not trust the
      // reamining/additonal-used fields anymore.  
      //
      // if (($matches[2]-$matches[4]+$matches[5])!=$matches[3])
      //    fail("plan-remaining+overage!=used");

      // add this line to our array.
      if (!array_key_exists($matches[1],$sprintdata))
	$sprintdata[$matches[1]]=$temp;
      else
	{
	  // we already had a line of minutes with this label (e.g., MINUTES IN PLAN)
	  // Just merge the two entries.
	  $sprintdata[$matches[1]]["plan"]+=$temp["plan"];
	  $sprintdata[$matches[1]]["used"]+=$temp["used"];
	}
    }

  //  $dateregex="(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)[a-z]*.[0-9]{1,2}(st|nd|rd|th)?,?.[0-9]{2,4}";
  $dateregex="(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)[a-z]*.[0-9]{1,2},?.[0-9]{2,4}";
  //  $undateregex="(([^JFMASOND])|(J[^au])|(F[^e])|(M[^a])|(A[^pu])|(S[^e])|(O[^c])|(N[^o])|(D[^e]))*";
  $timeregex="[0-9]{1,2}:[0-9]{2}( [A-Z]{3})?";
  $dayofweekregex="(Mon|Tue|Wed|Thu|Fri|Sat|Sun)[a-z]*,?";

  /*
  if (preg_match("/Start Date[^A-Z]*($dateregex)/",$response[$i],$m))
    {
      $starttime=strtotime($m[1]);
    }

  if (preg_match("/End Date[^A-Z]*($dateregex)/",$response[$i],$m))
    {
      $endtime=strtotime($m[1]);
    }
  */

  if (preg_match("/Current as of[^A-Z]*($dateregex)/",$response[$i],$m))
    {
      $currentasoftime=strtotime($m[1]);
    }

  if (preg_match("/Invoice period[^A-Z]*($dateregex)[^A-Z]*($dateregex)/",$response[$i], $m))
    {
      $starttime=strtotime($m[1]);
      $endtime=strtotime($m[3]);
    }

  /*
  if (preg_match("/Current as of[^A-Z]*$dayofweekregex ($dateregex) at ($timeregex)/",$response[$i],$m))
    {
      $currentasoftime=strtotime($date[2]." ".$date[5]);
    }
  */
}

//var_dump($sprintdata);

// We require that there is at least a MINUTES IN PLAN field, or
// we assume something bad happened. (SprintPCS down?)
if (!array_key_exists("MINUTES IN PLAN",$sprintdata))
  fail("Request has failed, didn't get a MINUTES IN PLAN field. SprintPCS down?");

/** if we got this far, assume success. */
print("OKAY\n");

if ($debug)
  print "<h2>Done. Parsed data follows.</h2>";

$todaydatafile=$basepath."today.txt";
$summarydatafile=$basepath."history.txt";

$a=array();
$a["minutes"]=$sprintdata;
$a["starttime"]=$starttime;
$a["currentasoftime"]=$currentasoftime;
$a["updatetime"]=time();

// make the end date the *end* of the day on the specified date.
$enddate=getdate($endtime);
$a["endtime"]=mktime(23,59,59,$enddate["mon"],$enddate["mday"],$enddate["year"]);

if ($debug)
{
  //  print "<a href=$_SERVER[HTTP_REFERER]>Back to previous page</a><br>";
  print "<pre>";
  var_dump($a);
  print "</pre>";
}

if (!$nowrite)
{
  $fd=fopen($todaydatafile,"r");
  $oldraw=fgets($fd,8192);
  fclose($fd);
  $old=unserialize($oldraw);
  if ($old["currentasoftime"]==$a["currentasoftime"])
    {
      print "Data unchanged, not writing.";
      return;
    }

  $fd=fopen($todaydatafile,"w");
  fwrite($fd,serialize($a));
  fclose($fd);
  
  $fd=fopen($summarydatafile,"a");
  fwrite($fd,serialize($a)."\n");
  fclose($fd);
}
?>
